/* ====================================================== */
/*                      XCFGFILE.CPP                      */
/*           (C) 1993 Thomas Wlfer & toolbox             */
/*   definiert das Klassenverhalten der Klasse CfgFile    */
/*           Compiler: Microsoft Visual C++ 1.0           */
/* ====================================================== */

#include <afxcoll.h>
#include "xcfgfile.h"
#include <ctype.h>
#include <stdlib.h>

/* ====================================================== */
/* Intern verwendete Klassen, die fr den Benutzer der    */
/*       Klasse XCfgFile nicht von Interesse sind       */
/* ------------------------------------------------------ */
/* XEntry:                                                */
/* Ein Eintrag im File (eine Zeile, die kein TAG ist)     */
/*                                                        */
/* Mehrere Memberfunktionen knnen sowohl                 */
/* CMemoryExceptions als auch CFileExceptions werfen.     */
/*                                                        */
/* Member:                                                */
/*   m_szEntry: Der Eintrag als String. Dies ist eine     */
/*              Zeile, die kein Tag ist.                  */
/*   m_iValue:  Index auf den Wert in diesem Eintrag oder */
/*              0. Falls dies 0 ist, ist der Eintrag ein  */
/*              Kommentar.                                */
/*                                                        */
/* GetValue() Lesen des Wertes aus dem Eintrag            */
/* SetValue() Setzen des Wertes des Eintrags              */
/* IsKey()    Test, ob der Key dieses Eintrags ein        */
/*            gesuchter Key ist.                          */
/* Fill()     Fllen eines Eintrags beim Lesen aus einer  */
/*            Datei                                       */
/* Add()      Setzen des Eintrags anhand von Key und      */
/*            Value. Diese Funktion wird von den          */
/*            Write...()-Funktionen verwendet, wenn in    */
/*            einem Tag ein neuer Eintrag angelegt werden */
/*            soll                                        */
/* Flush()    Schreibt diesen Eintrag in ein CFile.       */
/* ====================================================== */

class XEntry : public CObject
{
  CString    m_szEntry;
  int        m_iValue;

public:
  // Wert lesen
  void GetValue(CString& rszValue) const
  {
    ASSERT_VALID(this);
    ASSERT(m_iValue >= 0);
    ASSERT(&rszValue != NULL);
    rszValue = m_szEntry.Mid(m_iValue);
  }

  // Wert schreiben
  void SetValue(const char *pszValue)
  {
    ASSERT_VALID(this);
    ASSERT(m_iValue > 0);
    ASSERT(AfxIsValidAddress(pszValue, strlen(pszValue),
           FALSE));
    m_szEntry = m_szEntry.Left(m_iValue) + pszValue;
  }

  // Testen, ob es sich um einen Schlssel handelt
  BOOL IsKey(const char *pszKey) const
  {
    ASSERT_VALID(this);
    ASSERT(m_iValue >= 0);
    ASSERT(AfxIsValidAddress(pszKey, strlen(pszKey),
           FALSE));
    if (m_iValue)
    {
      return
      ! m_szEntry.Left(m_iValue - 1).CompareNoCase(pszKey);
    }
    else
      return FALSE;
  }

  void Fill(const char *pszOneLine)
  {
    ASSERT_VALID(this);
    ASSERT(AfxIsValidAddress(pszOneLine, strlen(pszOneLine),
           FALSE));
    m_szEntry = pszOneLine;
    m_iValue  = m_szEntry.Find('=') + 1;
  }

  // Eintrag hinzufgen
  void Add(const char *pszKey, const char *pszValue)
  {
    ASSERT_VALID(this);
    ASSERT(AfxIsValidAddress(pszKey, strlen(pszKey),
           FALSE));
    ASSERT(AfxIsValidAddress(pszValue, strlen(pszValue),
           FALSE));
    m_szEntry  = pszKey;
    m_szEntry += '=';
    m_iValue   = m_szEntry.GetLength();
    m_szEntry += pszValue;
  }

  // Zurckschreiben
  void Flush(CStdioFile& rDest) const
  {
    ASSERT_VALID(this);
    ASSERT(&rDest != NULL);
    ASSERT(m_iValue >= 0);
    rDest.WriteString(m_szEntry + '\n');
  }

  // Test ob dies ein Eintrag ist
  BOOL IsEntry() const
  {
    ASSERT_VALID(this);
    return m_iValue ? TRUE : FALSE;
  }

  // Schlssel erfragen
  void GetKey(CString& rszKey) const
  {
    ASSERT_VALID(this);
    ASSERT(&rszKey != NULL);
    rszKey = m_szEntry.Left(m_iValue - 1);
  }

#ifdef _DEBUG
  void AssertValid() const
  {
    CObject::AssertValid();
  }
#endif
};

/* ====================================================== */
/* XTag:   Ein Tag aus einer Datei                        */
/*                                                        */
/* Member: GetString()   Liest einen String aus dem Tag   */
/*         WriteString() Schreibt einen String in den Tag */
/*         Fill()        Wird beim Einlesen einer Datei   */
/*                       verwendet                        */
/*         Flush()       Schreibt den Tag in ein CFile    */
/*                       zurck                           */
/* ====================================================== */

class XTag : public CObList
{
  BOOL GetEntry(CString& rszEntry,
                BOOL fIsFirst = FALSE) const;

public:
  BOOL GetString(const char *szKey,
                 const char *szDefault, CString& szValue);
  void WriteString(const char *szKey, const char *szValue);
  void Fill(const char *szOneLine);
  void Flush(CStdioFile& dest)           const;

  BOOL GetFirstEntry(CString& rszEntry)  const;
  BOOL GetNextEntry(CString& rszEntry)   const;

  ~XTag();
#ifdef _DEBUG
   void AssertValid() const
   {
     CObList::AssertValid();
   }
#endif
};


/* ****************** [ Implementierung ] *************** */


/* ====================================================== */
/* XTag::Flush - Zurckschreiben eines Tags               */
/* Parms:        CStdioFile& dest - Zieldatei             */
/* Returns:      void                                     */
/* Exceptions:   CFileException                           */
/* ====================================================== */

void XTag::Flush(CStdioFile& rDest) const
{
  ASSERT_VALID(this);
  ASSERT(&rDest != NULL);

  POSITION pos = GetHeadPosition();
  while (pos != NULL)
  {
    XEntry *theEntry = (XEntry*)GetNext(pos);
    theEntry->Flush(rDest);
  }
}

/* ====================================================== */
/* XTag::GetString - Lesen eines Strings aus einem Tag    */
/*                   Ist der Key im Tag nicht vorhanden,  */
/*                   wird der Defaultwert verwendet.      */
/* Parms:      szKey     - gesuchter Key                  */
/*             szDefault - Defaultwert, falls Key nicht   */
/*                         gefunden                       */
/*             szValue   - Zielspeicher fr Wert des Keys */
/* Returns:    TRUE      - Wert gefunden                  */
/*             FALSE     - Defaultwert verwendet          */
/* Exceptions: CMemoryException                           */
/* ====================================================== */

BOOL XTag::GetString(const char *pszKey,
                     const char *pszDefault,
                     CString& rszValue)
{
  ASSERT_VALID(this);
  ASSERT(AfxIsValidAddress(pszKey, strlen(pszKey), FALSE));
  ASSERT(AfxIsValidAddress(pszDefault, strlen(pszDefault),
         FALSE));
  ASSERT(&rszValue != NULL);

  POSITION pos = GetHeadPosition();

  while (pos != NULL)
  {
    XEntry *theEntry = (XEntry*)GetNext(pos);

    if (theEntry->IsKey(pszKey))
    {
      theEntry->GetValue(rszValue);
      return TRUE;
    }
  }

  // Wenn der Key nicht gefunden wurde, Defaultwert nehmen

  rszValue = pszDefault;
  return FALSE;
}

/* ====================================================== */
/* XTag::WriteString - Schreiben eines Keys mit einem     */
/*                     Wert in einen Tag. Wird der Key    */
/*                     nicht gefunden, wird er angelegt.  */
/*                     Ist der Key bereits vorhanden,     */
/*                     wird sein alter Wert mit dem neuen */
/*                     berschrieben                      */
/*                     Ist der Wert NULL, wird der        */
/*                     gesamte Key gelscht               */
/* Parms:    szKey   - Der zu schreibende Key             */
/*           szValue - Der Wert des Keys                  */
/* Returns:  void                                         */
/* Exceptions: CFileException / CMemoryException          */
/* ====================================================== */

void XTag::WriteString(const char *pszKey,
                       const char *pszValue /* =NULL */)
{
  ASSERT_VALID(this);
  ASSERT(AfxIsValidAddress(pszKey, strlen(pszKey), FALSE));
  ASSERT(*pszKey!=NULL);

  if (pszValue == NULL)
  {
    POSITION pos1, pos2;

    for (pos1 = GetHeadPosition();
      (pos2 = pos1) != NULL;)
    {
      XEntry *pEntry = (XEntry*)GetNext(pos1);
      if (pEntry->IsKey(pszKey))
        RemoveAt(pos2);
    }
  }
  else
  {
    POSITION pos = GetHeadPosition();
    while (pos != NULL)
    {
      XEntry *pEntry = (XEntry*)GetNext(pos);
      if (pEntry->IsKey(pszKey))
      {
        pEntry->SetValue(pszValue);
        return;
      }
    }

    XEntry *pEntry = new XEntry;
    TRY
    {
      pEntry->Add(pszKey, pszValue);
      AddTail(pEntry);
    }
    CATCH(CMemoryException, e)
    {
      delete pEntry;
      THROW_LAST();
    }
    END_CATCH
  }
}

/* ====================================================== */
/* XTag::Fill - Schreiben einer Zeile aus einem File in   */
/*              einen Tag. Die Funktion wird              */
/*              ausschlielich in der Phase des           */
/*              Dateilesens verwendet.                    */
/* Parms:       szOneLine - Die zu schreibende Zeile      */
/* Returns:     void                                      */
/* Exceptions:  CMemoryException                          */
/* ====================================================== */

void XTag::Fill(const char *pszOneLine)
{
   ASSERT_VALID(this);
   ASSERT(AfxIsValidAddress(pszOneLine, strlen(pszOneLine), FALSE));

   XEntry *pEntry = new XEntry;
   TRY
   {
      pEntry->Fill(pszOneLine);
      AddTail(pEntry);
   }
   CATCH(CMemoryException, e)
   {
      delete pEntry;
      THROW_LAST();
   }
   END_CATCH
}

/* ====================================================== */
/* XTag::GetEntry       - Liefert einen Eintrag aus       */
/*                        diesem Tag                      */
/* Parsm:      rszTag   - Speicher fr den Tag            */
/*             fIsFirst - Flag, ob vorne angefangen       */
/*                        werden soll                     */
/* Returns:    TRUE     - Es gibt einen weitere Tag       */
/*             FALSE    - kein weitere Tag                */
/* Exceptions: CMemoryException                           */
/* ====================================================== */

BOOL
XTag::GetEntry(CString& rszEntry, BOOL fIsFirst) const
{
  static POSITION pos;

  if (fIsFirst) pos = GetHeadPosition();

  while (pos != NULL)
  {
    XEntry *pEntry = (XEntry*)GetNext(pos);

    if (pEntry->IsEntry())
    {
      pEntry->GetKey(rszEntry);
      return TRUE;
    }
  }
  return FALSE;
}

/* ====================================================== */
/* XTag::GetFirstTag - Liefert den 1. Entry aus diesem Tag*/
/* Parms:   rszEntry - Speicher fr den 1. Entry          */
/* Returns: TRUE     - Es gibt einen 1. Entry             */
/*          FALSE    - Es gibt keinen 1. Entry            */
/* Exceptions: CMemoryException                           */
/* ====================================================== */

BOOL XTag::GetFirstEntry(CString& rszEntry) const
{
  return GetEntry(rszEntry, TRUE);
}

/* ====================================================== */
/* XTag::GetNextEntry - Liefert den nchsten Entry aus    */
/*                      der Liste der Entries             */
/*                      in diesem Tag                     */
/* Parms:   rszEntry  - Speicher fr den Entry            */
/* Returns: TRUE      -  Es gibt einen weiteren Entry     */
/*          FALSE     - Kein weiterer Entry               */
/* Exceptions: CMemoryException                           */
/* ====================================================== */

BOOL XTag::GetNextEntry(CString& rszEntry) const
{
  return GetEntry(rszEntry);
}

/* ====================================================== */
/* XTag::~XTag - Zerstrt einen Tag.                      */
/*               Dabei werden alle Eintrge freigegeben.  */
/* ====================================================== */

XTag::~XTag()
{
  ASSERT_VALID(this);

  POSITION pos = GetHeadPosition();
  while (pos != NULL)
     delete (XEntry*)GetNext(pos);
  RemoveAll();
}

// may throw a CFileException

XCfgFile::XCfgFile(const char  *szFile)
{
  CStdioFile Src(szFile, CFile::modeRead);
  m_strFileName = szFile;
  char sz[256];
  while (Src.ReadString(sz, 255))
  {
    for (int iLen = strlen(sz) - 1; iLen >= 0; --iLen)
      if (! isspace(sz[iLen]))
        break;
    sz[ iLen+1 ] = '\0';
    Fill(sz);
  }
  m_fIsDirty = FALSE;
}

/* ====================================================== */
/* XCfgFile::IsTag - berprfen, ob ein String ein Tag    */
/*                   ist. Diese Funktion wird nur in der  */
/*                   Phase des File-Einlesens verwendet.  */
/*                   Damit eine Zeile als Tag erkannt     */
/*                   wird, mu sie in der 1. Spalte eine  */
/*                   [-Klammer und spter eine          */
/*                   ]-Klammer besitzen. Weiterhin mu  */
/*                   mindestens ein Non-Whitespace        */
/*                   zwischen den beiden Klammern stehen. */
/*                   Wenn es sich um einen Tag handelt,   */
/*                   wird er zurckgeliefert.             */
/* Parms:   szLine - Die zu berprfende Zeile            */
/*          szTag  - Speicher fr den zurckzulief. Tag   */
/* Returns: TRUE   - Zeile enthlt einen Tag              */
/*          FALSE  - kein Tag                             */
/* Exceptions: CMemoryException                           */
/* ====================================================== */

BOOL XCfgFile::IsTag(const CString& rszLine,
                           CString& rszTag) const
{
  ASSERT_VALID(this);
  ASSERT(&rszLine != NULL);
  ASSERT(&rszTag  != NULL);

   //-------------------------------------------------------
   // Wenn szLine leer ist, fhrt das zum Fehlschlagen
   // des Asserts weiter unten. Aus diesem Grund wird diese
   // Bedingung zuerst abgeprft.

   if (rszLine.IsEmpty())
   {
     return FALSE;
   }

   if (rszLine[0] == '[' && (rszLine.Find(']') != -1))
   {
     int iBegin=0;
     int iEnd=rszLine.Find(']');

     CString szTmp = rszLine.Mid(iBegin+1, iEnd-1);

     TRY
     {
        for (int iTag=0; iTag<szTmp.GetLength(); iTag++)
           if (! isspace(szTmp[ iTag ])) break;
        if (szTmp.GetLength() == iTag) return FALSE;
                          // Tag besteht nur aus Spaces ...
        while (szTmp[iBegin]   == ' ')  iBegin++;
        while (szTmp[iEnd - 2] == ' ')  iEnd--;
        rszTag = szTmp.Mid(iBegin, iEnd-1);
     }
     CATCH(CMemoryException, e)
     {
       szTmp.Empty();
       THROW_LAST();
     }
     END_CATCH
     return TRUE;
  }
  else
    return FALSE;
}

/* ====================================================== */
/* XCfgFile::AddTag - Hinzufgen eines Tags               */
/*                    Der Tag wird sowohl in die Map      */
/*                    eingetragen und zeigt dort auf eine */
/*                    Liste aus Eintrgen als auch in die */
/*                    Stringliste aus Tags. Diese Liste   */
/*                    wird fr das Zurckschreiben des    */
/*                    Files auf Platte in der korrekten   */
/*                    Reihenfolge bentigt.               */
/* Parms:   szTag   - Der neue Tag                        */
/* Returns: XTag*   - Zeiger auf den neuen Tag.           */
/* Exceptions: CMemoryException                           */
/* ====================================================== */

XTag *XCfgFile::AddTag(const char *pszTag)
{
  ASSERT_VALID(this);
  ASSERT(AfxIsValidAddress(pszTag, strlen(pszTag), FALSE));

  XTag *pTag = NULL;

  TRY
  {
    m_lstTags.AddTail(pszTag);
    pTag = new XTag;
    CString szTagUpper(pszTag);
    szTagUpper.MakeUpper();
    SetAt(szTagUpper, pTag);
  }
  CATCH(CMemoryException, e)
  {
    m_lstTags.RemoveTail();
    delete pTag;
    THROW_LAST();
  }
  END_CATCH
  return pTag;
}

/* ====================================================== */
/* XCfgFile::GetString - Lesen eines String aus dem File  */
/* Parms: szSection - Tag, aus dem gelesen werden soll    */
/*        szKey     - Key, der gelesen werden soll        */
/*        szDefault - Defaultwert, der verwendet werden   */
/*                    soll, falls der Key nicht gelesen   */
/*                    werden kann                         */
/*        szValue   - Speicher fr den Wert               */
/* Returns: FALSE   - Tag nicht gefunden                  */
/*          sonst   - Returnwert der verwendeten          */
/*                    Get...() Funktion                   */
/* Exceptions: CMemoryException                           */
/* ====================================================== */

BOOL XCfgFile::GetString(const char *pszSection,
                         const char *pszKey,
                         const char *pszDefault,
                         CString& rszValue)
{
  ASSERT_VALID(this);
  ASSERT(AfxIsValidAddress(pszSection, strlen(pszSection),
                           FALSE));
  ASSERT(AfxIsValidAddress(pszKey, strlen(pszKey), FALSE));
  ASSERT(AfxIsValidAddress(pszDefault, strlen(pszDefault),
                           FALSE));
  ASSERT(&rszValue != NULL);

  CObject *pOb;
  CString szSectionUpper = pszSection;

  szSectionUpper.MakeUpper();

  TRY
  {
    if (! Lookup(szSectionUpper, pOb))
    {
      rszValue = pszDefault;
      return FALSE;
    }
    else
    {
      XTag *pTag = (XTag*)pOb;
      return pTag->GetString(pszKey, pszDefault, rszValue);
    }
  }
  CATCH(CException, e)
  {
    szSectionUpper.Empty();
    THROW_LAST();
  }
  END_CATCH
  return FALSE;     // Obsolete, aber fr Compiler notwendig
}

/* ====================================================== */
/* XCfgFile::WriteString - Schreiben eines String in ein  */
/*                         File.                          */
/*                         Wenn die Sektion, in die       */
/*                         geschrieben werden soll nicht  */
/*                         nicht vorhanden ist, wird sie  */
/*                         angelegt.                      */
/*                         Wenn der Key == NULL ist, wird */
/*                         die Sektion gelscht           */
/*                         Das Dirty-Flag dieses Files    */
/*                         wird gesetzt, damit das File   */
/*                         bei der Zerstrung des Objekts */
/*                         noch vor der Freigabe          */
/*                         zurckgeschrieben wird.        */
/* Parms:    szSection -   Der Tag, in den geschrieben    */
/*                         werden soll                    */
/*           szKey     -   Der Key, der geschrieben       */
/*                         werden soll                    */
/*           szValue   -   Wert des Keys                  */
/* Returns:  void                                         */
/* Exceptions: CFileException / CMemoryException          */
/* ====================================================== */

void XCfgFile::WriteString(const char *pszSection,
                           const char *pszKey   /*==NULL*/,
                           const char *pszValue /*==NULL*/)
{
  ASSERT_VALID(this);
  ASSERT(AfxIsValidAddress(pszSection, strlen(pszSection),
                           FALSE));

  // Dafr sorgen da spter auch geflushed wird.
  m_fIsDirty = TRUE;

  CObject *pOb;
  XTag    *pTag;

  CString szSectionUpper = pszSection;
  szSectionUpper.MakeUpper();

  TRY
  {
    if (! Lookup(szSectionUpper, pOb))
      pTag = AddTag(pszSection);
    else
      pTag = (XTag*)pOb;

    if (pszKey == NULL)
    {
      delete pTag;
      RemoveKey(szSectionUpper);
      m_lstTags.RemoveAt(m_lstTags.Find(pszSection));
    }
    else
      pTag->WriteString(pszKey, pszValue);
  }

  CATCH(CException, e)
  {
    szSectionUpper.Empty();
    THROW_LAST();
  }
  END_CATCH
}

/* ====================================================== */
/* XCfgFile::Fill     - Ablegen einer gerade gelesenen    */
/*                      Zeile aus dem File                */
/* Parms:   szOneLine - Die Zeile                         */
/* Returns: void                                          */
/* Exceptions: CFileException / CMemoryException          */
/* Die CNotSupportedException wird geworfen, falls das    */
/*   Fileformat nicht ok ist.                             */
/* ====================================================== */

void XCfgFile::Fill(const char *pszOneLine)
{
  ASSERT_VALID(this);
  ASSERT(AfxIsValidAddress(pszOneLine, strlen(pszOneLine),
                           FALSE));
  CString szTag;
  CString szTagUpper;

  if (IsTag(pszOneLine, szTag))                 // Falls TAG
  {
    CObject *pOb;

    szTagUpper = szTag;
    szTagUpper.MakeUpper();

    if (Lookup(szTagUpper, pOb))
    {
      szTagUpper.Empty(); // Um TRY/CATCH Block zu vermeiden
      AfxThrowNotSupportedException();  // Dopp. Tag im File
    }
    else
    {
      szTagUpper.Empty(); // Um TRY/CATCH Block zu vermeiden
      m_pLastTag = AddTag(szTag);
    }
  }
  else                           // Eintrge oder Kommentare
  {
    if (IsEmpty())                // fhrende Kommentare
      m_lstLeadingComments.AddTail(pszOneLine);
    else
      m_pLastTag->Fill(pszOneLine);
  }
}

/* ====================================================== */
/* XCfgFile::GetTag -  Liefert einen Tag aus der Liste    */
/*                     der Tags in diesem File            */
/* Parsm:   rszTag   - Speicher fr den Tag               */
/*          fIsFirst - Flag, ob vorne angefangen werden   */
/*                     soll                               */
/* Returns: TRUE     - Es gibt einen weiteren Tag         */
/*          FALSE    - keine weiteren Tags                */
/* Exceptions: CMemoryException                           */
/* ====================================================== */

BOOL XCfgFile::GetTag(CString& rszTag, BOOL fIsFirst) const
{
  static POSITION pos;

  if (fIsFirst) pos = m_lstTags.GetHeadPosition();
  if(pos != NULL)
  {
    rszTag = m_lstTags.GetNext(pos);
    return TRUE;
  }
  return FALSE;
}

/* ====================================================== */
/* XCfgFile::GetFirstTag -  Liefert den 1. Tag aus diesem */
/*                          CfgFile                       */
/* Parms:      rszTag -     Speicher fr den 1. Tag       */
/* Returns:    TRUE   -     Es gibt einen 1. Tag          */
/*             FALSE  -     Es gibt keinen 1. Tag         */
/* Exceptions: CMemoryException                           */
/* ====================================================== */

BOOL XCfgFile::GetFirstTag(CString& rszTag) const
{
  return GetTag(rszTag, TRUE);
}

/* ====================================================== */
/* XCfgFile::GetNextTag - Liefert den nchsten Tag aus d. */
/*                        Liste der Tags in diesem File   */
/* Parms:   rszTag      - Speicher fr den Tag            */
/* Returns: TRUE        - Es gibt einen weiteren Tag      */
/*          FALSE       - Kein weiterer Tag               */
/* Exceptions: CMemoryException                           */
/* ====================================================== */

BOOL XCfgFile::GetNextTag(CString& rszTag) const
{
  return GetTag(rszTag);
}

/* ====================================================== */
/* XCfgFile::GetFirstEntry - Lesen des 1. Eintrags aus    */
/*                           einem Tag in diesem File     */
/* Parms:   pszTag   - der Tag aus dem gelesen werden soll*/
/*          rszEntry - Speicher fr den Eintrag           */
/* Returns: TRUE     - Es gibt in diesem Tag einen 1.     */
/*                     Eintrag                            */
/* Exceptions: CMemoryException                           */
/* ====================================================== */

BOOL XCfgFile::GetFirstEntry(const char *pszTag,
                             CString& rszEntry) const
{
  ASSERT_VALID(this);
  ASSERT(AfxIsValidAddress(pszTag, strlen(pszTag), FALSE));
  ASSERT(&rszEntry != NULL);

  CObject *pOb;
  CString szSectionUpper = pszTag;
  szSectionUpper.MakeUpper();

  TRY
  {
    if (! Lookup(szSectionUpper, pOb))
      return FALSE;
    else
    {
      XTag *pTag = (XTag*)pOb;
      return pTag->GetFirstEntry(rszEntry);
    }
  }

  CATCH(CException, e)
  {
    szSectionUpper.Empty();
    THROW_LAST();
  }
  END_CATCH
  return FALSE;     // Obsolete, aber fr Compiler notwendig
}

/* ====================================================== */
/* XCfgFile::GetNextEntry - Lesen des folgenden Eintrags  */
/*                          aus einem Tag in diesem File  */
/* Parms:   pszTag   - der Tag aus dem gelesen werden soll*/
/*          rszEntry - Speicher fr den Eintrag           */
/* Returns: TRUE     - Es gibt in diesem Tag einen        */
/*                     weiteren  Eintrag                  */
/* Exceptions: CMemoryException                           */
/* ====================================================== */

BOOL XCfgFile::GetNextEntry(const char *pszTag,
                            CString& rszEntry) const
{
  ASSERT_VALID(this);
  ASSERT(AfxIsValidAddress(pszTag, strlen(pszTag), FALSE));
  ASSERT(&rszEntry != NULL);

  CObject *pOb;
  CString szSectionUpper = pszTag;

  szSectionUpper.MakeUpper();

  TRY
  {
    VERIFY(Lookup(szSectionUpper, pOb));
    XTag *pTag = (XTag*)pOb;
    return pTag->GetNextEntry(rszEntry);
  }

  CATCH(CException, e)
  {
    szSectionUpper.Empty();
    THROW_LAST();
  }
  END_CATCH

  return FALSE;     // Obsolete, aber fr Compiler notwendig
}

/* ====================================================== */
/* XCfgFile::~XCfgFile - Zerstrt ein Objekt vom Typ      */
/*                       XCfgFile. Dazu werden alle       */
/*                       Leading Comments und alle Tags   */
/*                       inklusive ihrer Eintrge         */
/*                       freigegeben                      */
/* ====================================================== */

XCfgFile::~XCfgFile()
{
  m_lstLeadingComments.RemoveAll();
  m_lstTags.RemoveAll();

  POSITION pos = GetStartPosition();
  while (pos != NULL)
  {
    CObject *pOb;
    CString sz;
    GetNextAssoc(pos, sz, pOb);
    delete (XTag*)pOb;
  }
  RemoveAll();
}

/* ====================================================== */
/* XCfgFile::Flush - Schreibt ein XCfgFile aus dem        */
/*                   Speicher in das zugeordnete File auf */
/*                   der Platte zurck. Zuerst werden     */
/*                   alle fhrenden Kommentare und dann   */
/*                   der Name des Tags in [- und ]-   */
/*                   Klammern geschrieben. Daraufhin wird */
/*                   die Flush()-Funktion der XTags       */
/*                   aufgerufen, die alle Eintrge des    */
/*                   Tags rausschreibt.                   */
/* Parms:   rDest -  File, in das geschrieben werden soll */
/* Returns: void                                          */
/* Exceptions: CFileException                             */
/* ====================================================== */

void XCfgFile::Flush() const
{
  // nur dann flushen, wenn dies auch notwendig ist
  if (! m_fIsDirty) return;

  CStdioFile Dest(m_strFileName,
                  CFile::modeWrite|CFile::modeCreate);

  ASSERT_VALID(this);
  POSITION pos = m_lstLeadingComments.GetHeadPosition();
  while (pos != NULL)
    Dest.WriteString(m_lstLeadingComments.GetNext(pos)
                   + '\n');
  pos = m_lstTags.GetHeadPosition();
  while (pos != NULL)
  {
    CString sz = m_lstTags.GetNext(pos);
    Dest.WriteString('[' + sz + "]\n");
    sz.MakeUpper();
    CObject *pOb;
    Lookup(sz, pOb);
    XTag *pTag = (XTag*)pOb;
    pTag->Flush(Dest);
  }
}

/* ====================================================== */
/*                   Ende von XCFGFILE.CPP                */
