/* ----------------------------------------------------------------------- */
/*                                tscrgrp.cpp                              */
/*                                                                         */
/*                         (c) Raimond Reichert '93                        */
/*                                                                         */
/* ----------------------------------------------------------------------- */
#define Uses_ifpStream
#define Uses_ofpStream
#define Uses_TEvent
#define Uses_TKeys
#define Uses_TScrollGroup
#define Uses_TStreamableClass

#include <tscrgrp.h>

/* -----------------------------------------------------------------------
                                TScrollGroup
   ----------------------------------------------------------------------- */
// bounds gibt wie gewohnt den Raum an, den die Gruppe einnehmen darf, und
// aMaxSize enthlt die maximale, "virtuelle" Ausdehnung. aOwner wird nur
// dazu bentigt, die Scrollbalken einzufgen, was nur der Fall ist, falls
// die maximale Ausdehnung die tatschliche auch wirklich berschreitet.
// Das Ganze sieht leicht abstrahiert etwa so aus:
//
//       Ŀ
//                                     
//                              
//                                     
//                              
//                                     
//                   ۳            
//                       
//                                      
//                                      
//                                      
//       
//
//           size.x    
//
//                 maxSize.x            
//
// Der kleinere Ausschnitt ist der tatschlich belegte, in bounds angegebene,
// vom grsseren muss nur die Grsse bekannt sein (maxSize). Der maximale
// Bewegegungsraum, der daraus fr die Gruppe resultiert, ist daher
// maxSize.x-size.x bzw maxSize.y-size.y, und mit diesem Wert werden die
// Scrollbalken initialisiert. Die "Kltze" in der Abbildung stellen Views
// dar, von denen auch nur ein Teil auf dem Bildschirm sichtbar sein kann,
// wie angedeutet. handleEvent zeigt, wie das Spiel mit dem virtuellen Raum
// betrieben wird.
// -----------------------------------------------------------------------
TScrollGroup::TScrollGroup (TRect& bounds,
                            TPoint& aMaxSize,
                            TGroup* aOwner) :
    TGroup (bounds),
    hValue (0),
    vValue (0),
    hScrollBar (0),
    vScrollBar (0),
    maxSize (aMaxSize)
{
    TPoint oversize;
    if ((oversize.x = maxSize.x-size.x) > 0)
    {
        TRect hsbBounds (bounds.a.x, bounds.b.y, bounds.b.x, bounds.b.y+1);
        hScrollBar = new TScrollBar (hsbBounds);
        hScrollBar->setRange (0, oversize.x);
        aOwner->insert (hScrollBar);
    }
    if ((oversize.y = maxSize.y-size.y) > 0)
    {
        TRect vsbBounds (bounds.b.x, bounds.a.y, bounds.b.x+1, bounds.b.y);
        vScrollBar = new TScrollBar (vsbBounds);
        vScrollBar->setRange (0, oversize.y);
        aOwner->insert (vScrollBar);
    }
    // damit wir die Gruppe vom Rest unterscheiden knnen und ihre
    // Elemente auch pronto auf die Ereignisse reagieren:
    options |= ofFramed + ofFirstClick;
}

// -----------------------------------------------------------------------
// draw ruft TView::draw () auf, damit der Hintergrund gelscht wird, und
// anschliessend redraw (), da es sonst wegen der Pufferung des Gruppen-
// inhaltes zu farblich durchaus interessanten Darstellungen am Bildschirm
// kommt.
// -----------------------------------------------------------------------
void TScrollGroup::draw ()
{
    TView::draw ();
    redraw ();
}

// -----------------------------------------------------------------------
// getFirstSelectable liefert die erste View der Gruppe, die von last an
// miteinander verkettet ist, deren ofSelectable-Bit gesetzt ist, geht
// dabei aber in der Reihenfolge vor, in die Views in die Gruppe eingefgt
// worden sind, im Gegensatz zu firstThat, das aus diesem Grund nicht
// benutzt werden kann; wird von handleEvent bei kbTab bentigt.
// -----------------------------------------------------------------------
#pragma warn -rvl
static TView* getFirstSelectable (TView* last)
{
    TView* view = last;
    do
    {
        if (view->options & ofSelectable)
            return view;
        view = view->prev ();
    } while (view != last);
}
#pragma warn +rvl

// -----------------------------------------------------------------------
// Wird von handleEvent bei kbShiftTab aufgerufen, um in der umgekehrten
// Reihenfolge als der, in der eingefgt wurde, nach der ersten
// selektierbaren View zu suchen.
// -----------------------------------------------------------------------
#pragma argsused
static Boolean isSelectable (TView* view, void* nothing)
{
    if (view->options & ofSelectable)
        return True;
    else
        return False;
}

// -----------------------------------------------------------------------
// moveView verschiebt die View view um delta, das als Zeiger auf TPoint
// interpretiert wird; wird von handleEvent bentigt, um die Views der
// Gruppe zu verschieben.
// -----------------------------------------------------------------------
static void moveView (TView* view, void* delta)
{
    TRect bounds = view->getBounds ();
    bounds.a.x += ((TPoint*)delta)->x;
    bounds.a.y += ((TPoint*)delta)->y;
    bounds.b.x += ((TPoint*)delta)->x;
    bounds.b.y += ((TPoint*)delta)->y;
    view->changeBounds (bounds);
}

// -----------------------------------------------------------------------
// handleEvent ist das Kernstck von TScrollGroup. Falls es vom Rundruf
// cmScrollBarChanged ereilt wird und dieser von einem seiner Scrollbalken
// (hScrollBar oder vScrollBar) ausgesandt wurde, muss es die Views im
// virtuellen Raum verschieben. Die Differenz aus hValue (oder vValue) und
// der neuen Position hScrollBar->value (analog vScrollBar...) ergibt den
// Wert der Verschiebung (also neuer Wert - alter Wert). Da nun aber nicht
// das "Guckloch" der obigen Abbildung verschoben wird, sondern die Views,
// die sich im virtuellen Raum befinden, muss dieser Wert noch negiert
// werden - ansonsten laufen die Views in der verkehrten Richtung davon.
//
// Weiterhin muss TScrollGroup auf kbTab/ShiftTab reagieren, denn wenn das
// dem Owner, einem Fensternachkommen, berlassen wird, ruft dieser
// selectNext auf, was die Gruppe als Ganzes nicht sonderlich berhrt.
// Dazu wird zunchst innerhalb der Gruppe die nchste View selektiert,
// und anschliessend muss geprft werden, ob das nun die erste respektive
// letzte selektierbare View in der Gruppe ist. Falls ja, so wird das
// Ereignis nicht gelscht, so dass sich der Owner darum bemht,
// seine nchste oder vorhergehende View zu aktivieren;
// falls nein, so ist die Arbeit bereits getan und das Ereignis darf
// gelscht werden.
//
// Letzte Aufgabe von handleEvent besteht darin, kbCtrlLeft/Right an
// den horizontalen Scrollbalken und kbCtrlPgDn/Up als kbPgDn/Up an
// den vertikalen weiterzuleiten, damit auch Leute, die nur die Tastatur
// benutzen, in den Genuss von TScrollGroup kommen.
// -----------------------------------------------------------------------
void TScrollGroup::handleEvent (TEvent& event)
{
    TGroup::handleEvent (event);

    if (event.what == evBroadcast &&
        event.message.command == cmScrollBarChanged)
    {
        if (event.message.infoPtr == hScrollBar)
        {
            TPoint delta;
            delta.x = - (hScrollBar->value-hValue);  delta.y = 0;
            hValue = hScrollBar->value;
            forEach (moveView, &delta);
            drawView ();
            clearEvent (event);
        }
        else if (event.message.infoPtr == vScrollBar)
        {
            TPoint delta;
            delta.x = 0;  delta.y = - (vScrollBar->value-vValue);
            vValue = vScrollBar->value;
            forEach (moveView, &delta);
            drawView ();
            clearEvent (event);
        }
    }

    if (event.what == evKeyDown)
    {
        switch (event.keyDown.keyCode)
        {
            case kbTab:
                if (state & sfFocused)
                {
                    selectNext (False);
                    if (current != getFirstSelectable (last))
                        clearEvent (event);
                    makeViewVisible (current);
                }
                break;
            case kbShiftTab:
                if (state & sfFocused)
                {
                    selectNext (True);
                    if (current != firstThat (isSelectable, 0))
                        clearEvent (event);
                    makeViewVisible (current);
                }
                break;
        }

        if (hScrollBar)
            switch (event.keyDown.keyCode)
            {
                case kbCtrlLeft:
                case kbCtrlRight:
                    hScrollBar->handleEvent (event);
                    break;
                default:
                    break;
            }
        if (vScrollBar)
            switch (event.keyDown.keyCode)
            {
                case kbCtrlPgUp:
                    event.keyDown.keyCode = kbPgUp;
                    vScrollBar->handleEvent (event);
                    break;
                case kbCtrlPgDn:
                    event.keyDown.keyCode = kbPgDn;
                    vScrollBar->handleEvent (event);
                    break;
                default:
                    break;
            }
    }
}

// -----------------------------------------------------------------------
// makeViewVisible hat den Zweck, die aktuell fokussierte View der Gruppe
// auch in den sichtbaren Bereich zu verschieben, was bei der Fokusweiter-
// gabe mit TAB oder SHIFT-TAB nicht immer der Fall ist. Zunchst wird die
// linke obere Ecke in den sichtbaren Bereich verschoben, falls ntig, und
// anschliessend wird dasselbe fr die rechte untere Ecke durchgefhrt.
// Der Aufruf von xScrollBar->setValue sorgt dafr, dass TScrollGroup
// ber cmScrollBarChanged auch von der Vernderung erfhrt und sich in
// handleEvent korrekt darstellt.
// -----------------------------------------------------------------------
void TScrollGroup::makeViewVisible (TView* view)
{
    TPoint delta;
    delta.x = view->origin.x;
    delta.y = view->origin.y;

    if (delta.x > 0) delta.x = 0;
    if (delta.y > 0) delta.y = 0;

    if (delta.x == 0)
    {
        delta.x = current->origin.x + current->size.x - size.x;
        if (delta.x < 0) delta.x = 0;
    }
    if (delta.y == 0)
    {
        delta.y = current->origin.y + current->size.y - size.y;
        if (delta.y < 0) delta.y = 0;
    }

    if (delta.x || delta.y)
    {
        if (hScrollBar && delta.x)
          hScrollBar->setValue (delta.x + hValue);
        if (vScrollBar && delta.y)
          vScrollBar->setValue (delta.y + vValue);
    }
}

void* TScrollGroup::read (ipstream& is)
{
    TGroup::read (is);
    is >> hValue >> vValue >> maxSize >> hScrollBar >> vScrollBar;
    return this;
}

void TScrollGroup::write (opstream& os)
{
    TGroup::write (os);
    os << hValue << vValue << maxSize << hScrollBar << vScrollBar;
}

TStreamable *TScrollGroup::build (void)
{
    return new TScrollGroup (streamableInit);
}

const char * const TScrollGroup::name = "TScrollGroup";

TStreamableClass RScrollGroup (TScrollGroup::name,
                               TScrollGroup::build,
                               __DELTA (TScrollGroup));

