(* ====================================================== *)
(*                          VESA.PAS                      *)
(*         Unit zur Ansteuerung der VESA-Funktionen       *)
(*           (c) 1993 Peter Meerwald & toolbox            *)
(* ====================================================== *)
UNIT Vesa;

INTERFACE

USES Objects;

CONST

  (* Rckgabewerte *)
  veSupported             = $4F;
  veOk                    = $00;
  veFailed                = $01;
  veSuccessful            = $004F;

  (* Interrupt-Nummer *)
  VideoInt                = $10;

  (* Videomodi *)
  ve640x400x256           = $100;
  ve640x480x256           = $101;
  ve800x600x16            = $102;
  ve800x600x256           = $103;
  ve1024x768x16           = $104;
  ve1024x768x256          = $105;
  ve1280x1024x16          = $106;
  ve1280x1024x256         = $107;
  ve80x60                 = $108;
  ve132x25                = $109;
  ve132x43                = $10A;
  ve132x50                = $10B;
  ve132x60                = $10C;

  (* Funktionen *)
  veVesaFunction          = $4F;
  veReturnInformation     = $00;
  veReturnModeInformation = $01;
  veSetVesaMode           = $02;
  veReturnMode            = $03;
  veSaveRestoreVesaState  = $04;
  veVesaBankControl       = $05;
  veSetGetScanLineLength  = $06;
  veSetGetDisplayStart    = $07;

  vfReturnInformation     = $4F00;
  vfReturnModeInformation = $4F01;
  vfSetVesaMode           = $4F02;
  vfReturnVesaMode        = $4F03;
  vfSaveRestoreVesaState  = $4F04;
  vfVesaBankControl       = $4F05;
  vfSetGetScanLineLength  = $4F06;
  vfSetGetDisplayStart    = $4F07;

  (* Unterfunktionen *)
  veReturnStateBufferSize = $00;
  veSaveVesaState         = $01;
  veRestoreVesaState      = $02;

  veSelectVesaBank        = $00;
  veReturnVesaBank        = $01;

  veSetScanLineLength     = $00;
  veGetScanLineLength     = $01;

  veSetDisplayStart       = $00;
  veGetDisplayStart       = $01;

  (* Mode Attribute *)
  vmSupported             = $0001;
  vmExtendedInformation   = $0002;
  vmBIOSOutput            = $0004;
  vmColor                 = $0008;
  vmGraphics              = $0010;

  (* Window Attribute *)
  vwSupported             = $0001;
  vwReadableBit           = $0002;
  vwWriteableBit          = $0004;
  vwReadable              = $0003;
  vwWriteable             = $0005;
  vwAccessable            = $0007;

  (* States fr Save/Restore *)
  vsHardware              = $0001;
  vsBIOS                  = $0002;
  vsDAC                   = $0004;
  vsSVga                  = $0008;

  (* allgemeines *)
  VesaBufferSize          = $0100;
  ClearMemory             = $8000;
  StateBufferGranularity  = $0040;

  (* Vesa Information Buffer *)
TYPE
  PVesaVersion = ^tVesaVersion;
  tVesaVersion = RECORD
    CASE BOOLEAN OF
      TRUE  : (Major, Minor : BYTE);
      FALSE : (All          : WORD);
  END;

  pVideoModesTable = ^tVideoModesTable;
  tVideoModesTable = ARRAY [BYTE] OF WORD;

  pVesaInformation = ^tVesaInformation;
  tVesaInformation = RECORD
    Signature    : ARRAY [0..3] OF CHAR;
    Version      : tVesaVersion;
    OEMString    : pString;
    Capabilities : ARRAY [0..3] OF BYTE;
    VideoModes   : pVideoModesTable;
    Reserved     : ARRAY[0..237] OF BYTE;
  END;

  pModeInformation = ^tModeInformation;
  tModeInformation = RECORD

  (* zwingende Informationen *)

    Attributes         : WORD;
    WindowAAttributes,
    WindowBAttributes  : BYTE;
    WindowGranularity,
    WindowSize,
    WindowASegment,
    WindowBSegment     : WORD;
    WindowFunction     : Pointer;
    BytesPerLine,

  (* optionale Informationen *)

    Width,
    Height              : WORD;
    CharWidth,
    CharHeight,
    Planes,
    BitsPixel,
    Banks,
    MemoryModel,
    BankSize             : BYTE;

    Reserved             : ARRAY [0..226] OF BYTE;
  END;

VAR
  VesaInstalled : BOOLEAN;

  (* Vesa V1.0 Funktionen *)

  FUNCTION IsVesaInstalled: BOOLEAN;
    (* Prft, ob Vesa-Erweiterung vorhanden ist           *)

  FUNCTION ReturnVesaInformation(VAR Buffer) : BOOLEAN;
    (* Gibt Informationen zurck; tVesaInformation;       *)
    (* Signature mu 'VESA' sein                          *)

  FUNCTION IsModeSupported(Mode : WORD) : BOOLEAN;
    (* Prft, ob der angegebene Modus untersttzt wird     *)

  FUNCTION ReturnModeInformation(    Mode : WORD;
                                 VAR Buffer) : BOOLEAN;
    (* Gibt Informationen ber den angegebenen Modus      *)
    (* zurck; tModeInformation                           *)

  FUNCTION SetVesaMode(Mode : WORD) : BOOLEAN;
    (* Setzt den angegebenen Videomodus; ist das hchste  *)
    (* Bit gesetzt, wird der Bildspeicher nicht gelscht  *)

  FUNCTION ReturnVesaMode : WORD;
    (* Gibt den aktuellen Videomodus zurck               *)

  FUNCTION ReturnStateBufferSize(State : WORD) : WORD;
    (* Gibt die Anzahl der 64-Byte-Block zurck, die zum  *)
    (* Sichern bentigt werden                            *)

  FUNCTION SaveVesaState(    State : WORD;
                         VAR Buffer) : BOOLEAN;
    (* Sichert den Status der Grafikkarte                 *)

  FUNCTION RestoreVesaState(    State : WORD;
                            VAR Buffer) : BOOLEAN;
    (* Stellt den Status der Grafikkarte wieder her       *)

  FUNCTION SetVesaBank(Window   : BYTE;
                       Position : WORD) : BOOLEAN;
    (* Setzt die Position des Speicherfenster; Window 0/1 *)

  FUNCTION ReturnVesaBank(Window : BYTE) : WORD;
    (* Gibt die Position des Speicherfensters zurck;     *)
    (* Window 0/1                                         *)

  (* Vesa V1.1 Funktionen *)

  FUNCTION SetScanLineLength(Length : WORD) : BOOLEAN;

  FUNCTION ReturnScanLineLength(VAR BytesLine,
                                PixelsLine,
                                MaxLines : WORD) : BOOLEAN;

  FUNCTION SetDisplayStart(Column, Line : WORD) : BOOLEAN;

  FUNCTION GetDisplayStart(VAR Column,
                               Line : WORD) : BOOLEAN;

IMPLEMENTATION

  FUNCTION IsVesaInstalled: BOOLEAN; ASSEMBLER;
  ASM
    MOV     AX, vfReturnVesaMode
    INT     VideoInt
    CMP     AL, veSupported
    MOV     AL, FALSE
    JNE     @@NotInstalled
    MOV     AL, TRUE
  @@NotInstalled:
  END;

  FUNCTION ReturnVesaInformation(VAR Buffer) : BOOLEAN;
    ASSEMBLER;
  ASM
    MOV     AX, vfReturnInformation
    LES     DI, Buffer
    INT     VideoInt
    CMP     AX, veSuccessful
    MOV     AL, FALSE
    JNE     @@Failed
    MOV     AL, TRUE
  @@Failed:
  END;

  FUNCTION IsModeSupported(Mode : WORD) : BOOLEAN;
    ASSEMBLER;
  VAR
    Buffer : ARRAY [0..255] OF BYTE;
  ASM
    PUSH    DS
    MOV     AX, vfReturnInformation
    PUSH    SS
    POP     ES
    LEA     DI, Buffer
    INT     VideoInt
    CMP     AX, veSuccessful
    MOV     AL, FALSE
    JNE     @@Failed
    LDS     SI, tVesaInformation(Buffer).VideoModes
  @@Next:
    LODSW
    CMP     AX, $FFFF
    JE      @@Failed
    CMP     AX, Mode
    JNE     @@Next
    MOV     AL, TRUE
  @@Failed:
    POP     DS
  END;

  FUNCTION ReturnModeInformation(Mode : WORD;
                                 VAR Buffer) : BOOLEAN;
    ASSEMBLER;
  ASM
    MOV     AX, vfReturnModeInformation
    LES     DI, Buffer
    MOV     CX, Mode
    INT     VideoInt
    CMP     AX, veSuccessful
    MOV     AL, FALSE
    JNE     @@Failed
    MOV     AL, TRUE
  @@Failed:
  END;

  FUNCTION SetVesaMode(Mode : WORD) : BOOLEAN; ASSEMBLER;
  ASM
    MOV     AX, vfSetVesaMode
    MOV     BX, Mode
    INT     VideoInt
    CMP     AX, veSuccessful
    MOV     AL, FALSE
    JNE     @@Failed
    MOV     AL, TRUE
  @@Failed:
  END;

  FUNCTION ReturnVesaMode : WORD; ASSEMBLER;
  ASM
    MOV     AX, vfReturnVesaMode
    INT     VideoInt
    CMP     AX, veSuccessful
    JNE     @@Failed
    MOV     AX, BX
  @@Failed:
  END;

  FUNCTION ReturnStateBufferSize(State : WORD) : WORD;
    ASSEMBLER;
  ASM
    MOV     AX, vfSaveRestoreVesaState
    MOV     DL, veReturnStateBufferSize
    MOV     CX, State
    INT     VideoInt
    CMP     AX, veSuccessful
    JNE     @@Failed
    MOV     AX, BX
  @@Failed:
  END;

  FUNCTION SaveVesaState(State : WORD;
                         VAR Buffer) : BOOLEAN; ASSEMBLER;
  ASM
    MOV     AX, vfSaveRestoreVesaState
    MOV     DL, veSaveVesaState
    MOV     CX, State
    LES     BX, Buffer
    INT     VideoInt
    CMP     AX, veSuccessful
    MOV     AL, FALSE
    JNE     @@Failed
    MOV     AL, TRUE
  @@Failed:
  END;

  FUNCTION RestoreVesaState(State : WORD;
                            VAR Buffer) : BOOLEAN;
  ASSEMBLER;
  ASM
    MOV     AX, vfSaveRestoreVesaState
    MOV     DL, veRestoreVesaState
    MOV     CX, State
    LES     BX, Buffer
    INT     VideoInt
    CMP     AX, veSuccessful
    MOV     AL, FALSE
    JNE     @@Failed
    MOV     AL, TRUE
  @@Failed:
  END;

  FUNCTION SetVesaBank(Window : BYTE;
                       Position : WORD) : BOOLEAN;
  ASSEMBLER;
  ASM
    MOV     AX, vfVesaBankControl
    MOV     BH, veSelectVesaBank
    MOV     BL, Window
    MOV     DX, Position
    INT     VideoInt
    CMP     AX, veSuccessful
    MOV     AL, FALSE
    JNE     @@Failed
    MOV     AL, TRUE
  @@Failed:
  END;

  FUNCTION ReturnVesaBank(Window : BYTE) : WORD; ASSEMBLER;
  ASM
    MOV     AX, vfVesaBankControl
    MOV     BH, veReturnVesaBank
    MOV     BL, Window
    INT     VideoInt
    CMP     AX, veSuccessful
    JNE     @@Failed
    MOV     AX, DX
  @@Failed:
  END;

  FUNCTION SetScanLineLength(Length : WORD) : BOOLEAN;
    ASSEMBLER;
  ASM
    MOV     AX, vfSetGetScanLineLength
    MOV     BL, veSetScanLineLength
    MOV     CX, Length
    INT     VideoInt
    CMP     AX, veSuccessful
    MOV     AL, FALSE
    JNE     @@Failed
    MOV     AL, TRUE
  @@Failed:
  END;

  FUNCTION ReturnScanLineLength(VAR BytesLine,
                                PixelsLine,
                                MaxLines : WORD) : BOOLEAN;
  ASSEMBLER;
  ASM
    MOV     AX, vfSetGetScanLineLength
    MOV     BL, veGetScanLineLength
    INT     VideoInt
    CMP     AX,veSuccessful
    MOV     AL, FALSE
    JNE     @@Failed
    LES     DI, BytesLine
    MOV     ES:[DI], BX
    LES     DI, PixelsLine
    MOV     ES:[DI], CX
    LES     DI, MaxLines
    MOV     ES:[DI], DX
    MOV     AL, TRUE
  @@Failed:
  END;

  FUNCTION SetDisplayStart(Column, Line : WORD) : BOOLEAN;
    ASSEMBLER;
  ASM
    MOV     AX, vfSetGetDisplayStart
    MOV     BH, 0
    MOV     BL, veSetDisplayStart
    MOV     CX, Column
    MOV     DX, Line
    INT     VideoInt
    CMP     AX, veSuccessful
    MOV     AL, FALSE
    JNE     @@Failed
    MOV     AL, TRUE
  @@Failed:
  END;

  FUNCTION GetDisplayStart(VAR Column,
                               Line : WORD) : BOOLEAN;
    ASSEMBLER;
  ASM
    MOV     AX, vfSetGetDisplayStart
    MOV     BH, 0
    MOV     BL, veGetDisplayStart
    INT     VideoInt
    CMP     AX, veSuccessful
    MOV     AL, FALSE
    JNE     @@Failed
    LES     DI, Column
    MOV     ES:[DI], CX
    LES     DI, Line
    MOV     ES:[DI], DX
    MOV     AL, TRUE
  @@Failed:
  END;

BEGIN
  VesaInstalled := IsVesaInstalled;
END.
(* ====================================================== *)
(*                     Ende von VESA.PAS                  *)

